Public Class PCDService
    Implements IDisposable

    'Variabili globali di Pcdmis con Gestione Eventi
    Private PcdApp As PCDLRN.Application
    Private PcdParts As PCDLRN.PartPrograms

    Private WithEvents PcdEvents As PCDLRN.ApplicationObjectEvents
    Private WithEvents PcdMachine As PCDLRN.Machine
    Private m_CheckError As Boolean
    Private m_CheckCancel As Boolean
    Private EndExec As Boolean

    Private RunCount As Integer     'Contatore del numero di esecuzioni

    Friend CountOOT As Integer      'Variabile che conta i valori fuori tolleranza
    Friend ExecHwnd As Integer      'Identificativo della finestra di esecuzione per gestire PostMessage


    'Delegate per messaggi di debug
    Delegate Sub DisplayMessage(ByVal Msg As String)
    Private LogMsg As DisplayMessage


    Function StartPCD() As Boolean
        'Avvia un'istanza dell'oggetto PCDApp

        ExecHwnd = 0

        LogMsg = New DisplayMessage(AddressOf WriteToStato)
        Dim CMMReady As Boolean
        Try
            If IsNothing(PcdApp) Then
                'FrmLoading.Show()
                Dim Pcd As String = CStr(My.Settings.PCDVersion)
                LogMsg.Invoke("Avviamento PC-Dmis in corso")
                PcdApp = CreateObject(Pcd)
                PcdEvents = PcdApp.ApplicationEvents
                PcdParts = PcdApp.PartPrograms
                'FrmLoading.Close()
                Application.DoEvents()

                RunCount = 0

            End If

            PcdApp.Minimize()
            PcdApp.Visible = My.Settings.PCDmisVisible
            PcdApp.OperatorMode = My.Settings.PCDmisOperatorMode

            Application.DoEvents()
            LogMsg.Invoke("Attesa CMM Ready")
            CMMReady = PcdApp.WaitUntilReady(60)
            If CMMReady = False Then
                LogMsg.Invoke("Errore di inizializzazione CMM. Verificare se esistono processi in sospeso e riprovare.")
                'MsgBox("Errore di inizializzazione CMM", MsgBoxStyle.Critical + MsgBoxStyle.MsgBoxSetForeground, "Errore Pc-Dmis")
                Exit Try
            Else
                LogMsg.Invoke("PC-Dmis Pronto")
            End If
            Return True
        Catch ex As ApplicationException
            LogMsg.Invoke(ex.Message)
            Return False
        End Try
    End Function
    Public Function CheckPCD() As Boolean
        'Verifica se  in esecuzione l'applicazione Pcdmis
        LogMsg = New DisplayMessage(AddressOf WriteToStato)

        Try
            Dim Proc() As Process = Process.GetProcessesByName("PCDLRN")
            If Proc.Length >= 1 Then
                LogMsg.Invoke("Processo PC-Dmis in esecuzione")
                Return True
            Else
                LogMsg.Invoke("Nessun Processo PC-Dmis in esecuzione")
                Return False
            End If
        Catch ProcessErr As SystemException
            'TODO: Gestire le eccezioni
            LogMsg.Invoke("CheckPCD_Error: " & ProcessErr.Message.ToString)
        End Try
    End Function

    Public Function KillPCD(ByVal BruteForce As Boolean) As Boolean
        'Verifica se  in esecuzione l'applicazione Pcdmis e la termina
        LogMsg = New DisplayMessage(AddressOf WriteToStato)

        Try
            Dim Proc() As Process = Process.GetProcessesByName("PCDLRN")
            If Proc.Length >= 1 Then
                LogMsg.Invoke("Il processo PC-Dmis in esecuzione sar terminato")
                Select Case BruteForce
                    Case False
                        Do While Proc.Length = 0
                            Dim ProcIndex = Proc.Length
                            KillPCD = Proc(ProcIndex - 1).CloseMainWindow
                            Proc(ProcIndex - 1).Close()
                            Me.Dispose()
                            GC.Collect()
                        Loop
                        LogMsg.Invoke("Processo PC-Dmis Terminato")
                    Case True
                        Do While Proc.Length = 0
                            Dim ProcIndex = Proc.Length
                            Proc(ProcIndex - 1).Kill()
                        Loop
                        LogMsg.Invoke("Processo PC-Dmis Terminato con Brute-Force")
                End Select
                Return True
            Else
                LogMsg.Invoke("Nessun Processo PC-Dmis in esecuzione")
                Return False
            End If
        Catch ProcessErr As SystemException
            'TODO: Gestire le eccezioni
            LogMsg.Invoke("KillPCD_Error: " & ProcessErr.Message.ToString)
        End Try
    End Function
    'TODO: Gestire esecuzione singola
    Overloads Function RunProgram() As Boolean
        Dim PcdPart As PCDLRN.PartProgram
        Try
            PcdPart = PcdApp.ActivePartProgram
            PcdPart.EXECUTE()
        Catch ex As Exception
            LogMsg.Invoke(ex.Message.ToString)
        End Try
    End Function

    Overloads Function RunProgram(ByRef Task As TaskItem) As Boolean
        Dim Pcdpart As PCDLRN.PartProgram
        'Richiama i valori di default dalle impostazioni dell'applicazione
        Dim TempDir As String = My.Settings.TempDir
        Dim tempAttrezzo As String = String.Empty
        Dim CadFileNAme As String = String.Empty


        'If RunCount >= 10 Then
        '    LogMsg.Invoke(String.Format("Effettuate {0} misure. PC-Dmis sar riavviato.", RunCount))

        '    Me.PCDStop()
        '    Me.KillPCD(False)
        '    Me.StartPCD()
        'End If

        Try

            LogMsg.Invoke(String.Format("Inizio misura di {0}", Task.DescrizioneProgramma))

            'Inizializza le variabili per il moitoraggio dei valori fuori tolleranza e delle richiesta di interruzione esecuzione
            CountOOT = 0
            m_CheckCancel = False
            m_CheckError = False
            EndExec = False

            'Copia il programma nella cartella di esecuzione temporanea
            Dim tempFile As String = String.Format("{0}\{1}", TempDir, Task.FileName)
            FileIO.FileSystem.CopyFile(Task.FileProgramma, tempFile, True)
            File.SetAttributes(tempFile, FileAttributes.Normal)

            LogMsg.Invoke(String.Format("File {0} copiato in {1}", Task.FileProgramma, TempDir))

            If Task.CadFile <> "" Then
                CadFileNAme = Task.CadFile
            Else
                CadFileNAme = Left(Task.FileProgramma, Len(Task.FileProgramma) - 4) & ".cad"
            End If

            If FileIO.FileSystem.FileExists(CadFileNAme) Then
                Dim tmpCadFileName As String = String.Format("{0}\{1}.cad", TempDir, Left(Task.FileName, Len(Task.FileName) - 4))
                FileIO.FileSystem.CopyFile(CadFileNAme, tmpCadFileName, True)
                LogMsg.Invoke(String.Format("File {0} copiato in {1}", CadFileNAme, TempDir))
            End If

            'Copia il file .aln con il nome standard e nella cartella predefinita
            If Task.DescrizioneAttrezzo <> "Macchina" Then
                If Task.NomeAllEsterno <> "" Then
                    tempAttrezzo = String.Format("{0}\{1}.aln", My.Settings.PcdDirAll, Task.NomeAllEsterno)
                Else
                    tempAttrezzo = String.Format("{0}\{1}.aln", My.Settings.PcdDirAll, My.Settings.PcdAllName & ".aln")
                End If

                FileIO.FileSystem.CopyFile(Task.FileAttrezzo, tempAttrezzo, True)
                LogMsg.Invoke(String.Format("File {0} copiato in {1}", Task.FileAttrezzo, tempAttrezzo))
            End If

            'Apre ed Esegue il programma selezionato, chiudendo prima tutti gli eventuali programmi aperti
            PcdParts.CloseAll()
            'RunCount += 1

            System.Threading.Thread.Sleep(1000)
            LogMsg.Invoke("Apertura File " & Task.FileProgramma)
            Pcdpart = PcdParts.Open(tempFile, "CMM1")
            PcdApp.WaitUntilReady(600)

            'Verifica opzioni part program
            Dim ds As New EasyRun.EasyRunDataSet
            Dim ta As New EasyRunDataSetTableAdapters.tbProgrammiTableAdapter
            Dim tb As New EasyRun.EasyRunDataSet.tbProgrammiDataTable
            ta.Fill(tb)

            Dim ActPrg As EasyRunDataSet.tbProgrammiRow = tb.FindByIdProgrammi(Task.IdProgramma)


            'Sovrascrivi opzioni di stampa
            If ActPrg.Stampa >= 0 Then
                Dim ReportDir As String = String.Format("{0}\{1}\", My.Settings.DirArchivioMis, Path.GetFileNameWithoutExtension(ActPrg.NomeFile))
                If Not Directory.Exists(ReportDir) Then Directory.CreateDirectory(ReportDir)
                Dim ReportName As String

                'If ActPrg.ReportName IsDBNull Then
                '    If Not String.IsNullOrEmpty(ActPrg.ReportName) Then
                'ReportName = String.Format("{0}{1}_{2}", ReportDir, ActPrg.ReportName, Task.MisID)
                'Else
                ReportName = String.Format("{0}{1}", ReportDir, Task.MisID)
                'End If
                '    End If

                Pcdpart.EditWindow.SetPrintOptionsEx(ActPrg.Stampa, PCDLRN.ENUM_PCD_ON_OFF.DMIS_OFF, PCDLRN.PCDPRINTFILEMODE.PCD_OVERWRITE, -1, ReportName, ActPrg.TipoReport, False)

            Else
                Pcdpart.EditWindow.SetPrintOptionsEx(PCDLRN.PCDPRINTLOC.PCD___OFF, PCDLRN.ENUM_PCD_ON_OFF.DMIS_OFF, PCDLRN.PCDPRINTFILEMODE.PCD_OVERWRITE, -1, "", PCDLRN.PCDFILEPRINTFORMAT.PCD_RTF, False)

            End If

            'Modifica Intestazione Programma
            Pcdpart.PartName = ActPrg.Descrizione
            Pcdpart.RevisionNumber = ActPrg.Versione
            Pcdpart.SerialNumber = Task.Id

            Application.DoEvents()
            LogMsg.Invoke("Inizio esecuzione " & Task.FileProgramma)

            If Not CheckCancel Then Pcdpart.EXECUTE()
            Do
                System.Threading.Thread.Sleep(100)
                Application.DoEvents()
            Loop Until EndExec
            LogMsg.Invoke("Esecuzione terminata " & Task.FileProgramma)

            PcdApp.WaitUntilReady(60)

            If My.Settings.ImportPCDMeasure AndAlso Not CheckError Then

                GetMeasValue(Task)

            End If

            ExecHwnd = 0

            PcdParts.CloseAll()
            System.Threading.Thread.Sleep(100)

            'Directory di salvataggio file prg dopo esecuzione
            If ActPrg.SalvaPartProgram Then

                Dim FileDir As String = String.Format("{0}\{1}\Prg\", My.Settings.DirArchivioMis, Path.GetFileNameWithoutExtension(tempFile))

                If Not Directory.Exists(FileDir) Then Directory.CreateDirectory(FileDir)
                Dim NewPath As String = String.Format("{0}{1}{2}{3}", FileDir, Path.GetFileNameWithoutExtension(tempFile), Task.MisID, Path.GetExtension(tempFile))

                If File.Exists(NewPath) Then
                    Dim i As Integer
                    Dim TryPath As String
                    Do
                        i += 1
                        TryPath = String.Format("{0}{1}_{2}{3}", FileDir, Path.GetFileNameWithoutExtension(NewPath), i, Path.GetExtension(NewPath))
                    Loop While File.Exists(TryPath) = True
                    File.Move(tempFile, TryPath)
                    LogMsg.Invoke(String.Format("File {0} archiviato in {1}", tempFile, TryPath))

                Else
                    File.Move(tempFile, NewPath)
                    LogMsg.Invoke(String.Format("File {0} archiviato in {1}", tempFile, NewPath))

                End If
            End If

            Dim tempCadPathFile As String = String.Format("{0}\{1}.cad", TempDir, Left(Task.FileName, Len(Task.FileName) - 4))
            If File.Exists(tempCadPathFile) Then File.Delete(tempCadPathFile)
            LogMsg.Invoke(String.Format("File Cad {0} eliminato", tempCadPathFile))

            If Not String.IsNullOrEmpty(tempAttrezzo) Then
                If FileIO.FileSystem.FileExists(tempAttrezzo) Then FileIO.FileSystem.DeleteFile(tempAttrezzo)
                LogMsg.Invoke(String.Format("File allineamento {0} eliminato", tempAttrezzo))
            End If

            Return CheckCancel

        Catch CastEx As InvalidCastException
            LogMsg.Invoke(CastEx.Message.ToString)
        Catch NullArgEx As NullReferenceException
            LogMsg.Invoke(NullArgEx.Message.ToString)
        Catch FileNotFound_EX As IO.FileNotFoundException
            LogMsg.Invoke(FileNotFound_EX.Message.ToString)
        Catch ComEx As System.Runtime.InteropServices.COMException
            LogMsg.Invoke(ComEx.Message.ToString)
            MsgBox("Impossibile comunicare con PC-Dmis. EasyRun verr chiuso", MsgBoxStyle.Critical, "Errore Grave")
            Application.Exit()

        Catch ex As Exception
            LogMsg.Invoke(ex.Message.ToString)

        End Try
    End Function

    Private Sub PcdEvents_OnClosePartProgram(ByVal PartProg As PCDLRN.PartProgram) Handles PcdEvents.OnClosePartProgram
        SetForegroundWindow(mdiMain.Handle)
    End Sub

    Private Sub PcdEvents_OnEndExecution(ByVal PartProg As PCDLRN.PartProgram, ByVal TerminationType As Integer) Handles PcdEvents.OnEndExecution
        Select Case TerminationType
            Case 1
                'Esecuzione terminata senza errori
                LogMsg.Invoke("Esecuzione Terminata correttamente")
                m_CheckCancel = False
                m_CheckError = False
            Case 2
                'Esecuzione terminata causa interruzione dell'operatore

                If MsgBox("Vuoi interrompere la coda in esecuzione?", MsgBoxStyle.YesNo + MsgBoxStyle.MsgBoxSetForeground, "Esecuzione annullata") = MsgBoxResult.Yes Then
                    m_CheckError = True
                    m_CheckCancel = True
                    LogMsg.Invoke("Esecuzione Annullata dall'operatore. Coda interrotta.")
                Else
                    m_CheckError = True
                    m_CheckCancel = False
                    LogMsg.Invoke("Esecuzione Annullata dall'operatore. Coda non interrotta.")
                End If
        End Select
        EndExec = True
    End Sub
    Public Sub PCDStop()
        If Not IsNothing(PcdMachine) Then PcdMachine = Nothing
        If Not IsNothing(PcdEvents) Then PcdEvents = Nothing
        If Not IsNothing(PcdParts) Then PcdParts = Nothing
        If Not IsNothing(PcdApp) Then
            PcdApp.Quit()
            PcdApp = Nothing
        End If

    End Sub
    Public Sub PCDStopRun()

        If ExecHwnd <> 0 Then
            PostMessage(ExecHwnd, &H111, 2, 0)
        End If

        If Not IsNothing(LogMsg) Then
            LogMsg.Invoke("Esecuzione interrotta dall'utente.")
        End If

    End Sub
    Private disposedValue As Boolean = False
    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: liberare le risorse non gestite chiamate in modo esplicito
                Me.PCDStop()
            End If
            ' TODO: liberare le risorse non gestite condivise

        End If
        Me.disposedValue = True
        LogMsg.Invoke("PC-Dmis Disposed")
    End Sub
#Region " IDisposable Support "
    ' Questo codice  aggiunto da Visual Basic per implementare in modo corretto il modello Disposable.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Non modificare questo codice. Inserire il codice di pulitura in Dispose(ByVal disposing As Boolean).
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
    Public Property CheckCancel() As Boolean
        Get
            Return m_CheckCancel
        End Get
        Set(ByVal value As Boolean)
            m_CheckCancel = value
        End Set
    End Property
    Public ReadOnly Property CheckError() As Boolean
        Get
            Return m_CheckError
        End Get
    End Property
#End Region

    Private Sub PcdMachine_ErrorMsg(ByVal Msg As String, ByVal ErrorType As Integer) Handles PcdMachine.ErrorMsg
        LogMsg.Invoke(Msg)
    End Sub

    Private Sub PcdEvents_OnObjectExecuted(ByVal PartProg As PCDLRN.PartProgram, ByVal Cmmd As PCDLRN.Command) Handles PcdEvents.OnObjectExecuted
        If Not String.IsNullOrEmpty(Cmmd.ID) Then

            If Cmmd.IsDimension Then
                CountOOT += Cmmd.DimensionCommand.OutTol <> 0

            Else
                LogMsg.Invoke(String.Format("{0}:{1}", Cmmd.TypeDescription, Cmmd.ID))
            End If

        End If

    End Sub

    Private Sub PcdEvents_OnOpenRemotePanelDialog(ByVal PartProg As PCDLRN.PartProgram, ByVal DialogId As Integer, ByVal hWnd As Integer, ByVal Message As String, ByVal BtnId1 As Integer, ByVal BtnId2 As Integer, ByVal BtnId3 As Integer, ByVal BtnId4 As Integer, ByVal DefaultBtn As Integer) Handles PcdEvents.OnOpenRemotePanelDialog
        'Identifica l'hWnd della finestra di esecuzione

        PcdApp.SetActive()

        If DialogId = 338 Then
            LogMsg.Invoke(String.Format("Finestra di esecuzione aperta. Handler: {0}", hWnd))
            ExecHwnd = hWnd
        End If

    End Sub

    Friend Function ChangePCDmisIsVisible(Optional ByVal ForceVisible As Boolean = False)
        If Not IsNothing(PcdApp) Then

            If ForceVisible Then
                PcdApp.Visible = True
            Else
                PcdApp.Visible = Not PcdApp.Visible
            End If

            LogMsg.Invoke(String.Format("Stato visibilit PC-Dmis = {0}", PcdApp.Visible.ToString))

            Return PcdApp.Visible

        Else
            If ForceVisible Then

                PcdApp = CreateObject("Pcdlrn.Application")
                PcdApp.Visible = True

            End If

            Return False
            End If
    End Function

    Friend Function GetPcdVersion() As String
        If Not IsNothing(PcdApp) Then
            Return PcdApp.VersionString
        Else
            Return String.Empty
        End If
    End Function

    Sub GetMeasValue(ByRef Task As TaskItem)

        Dim Cmds As PCDLRN.Commands = PcdApp.ActivePartProgram.Commands
        Dim Cmd As PCDLRN.Command

        Dim DimID As String     'Nome dimensione

        Dim MisId As String = Task.MisID
        Dim LastDim As String = ""
        Dim LineNum As Integer = 0


        LogMsg.Invoke(String.Format("{0}: {1} Importazione risultati di misura dal PartProgram {2}.", Date.Now, Task.FileProgramma, ControlChars.CrLf))

        Dim dt As DataTable = New EasyRunDataSet.tbValoriDataTable
        Dim dr As DataRow

        Try
            'Importa le righe del file Header
            Dim sr As New StreamReader("C:\Easyrun.net\header.txt")

            Do Until sr.EndOfStream
                Dim txt As String = sr.ReadLine

                If Not String.IsNullOrEmpty(txt) Then
                    LineNum += 1

                    dr = dt.NewRow

                    dr.Item(1) = MisId
                    dr.Item(2) = LineNum
                    dr.Item(3) = "Commento"
                    dr.Item(4) = txt.Replace("'", "")
                    dr.Item(5) = 0
                    dr.Item(6) = 0
                    dr.Item(7) = 0
                    dr.Item(8) = 0
                    dr.Item(9) = 0
                    dr.Item(10) = 0
                    dr.Item(11) = 0
                    dr.Item(12) = False
                    dr.Item(13) = ""
                    dr.Item(14) = True
                    dr.Item(15) = "Header Field"

                    dt.Rows.Add(dr)
                End If
            Loop

            sr.Close()

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

        'Importa i valori dal part program
        Try
            For I As Integer = 1 To Cmds.Count

                Cmd = Cmds.Item(I)

                'Se CMD  un commento
                If Cmd.IsComment Then

                    Dim txt As String = ""
                    If Cmd.CommentCommand.CommentType = PCDLRN.ENUM_PCD_COMMENT_TYPES.PCD_COMMENT_REPORT Then
                        txt = String.Format("{0}", Cmd.CommentCommand.Comment)
                    End If
                    If Cmd.CommentCommand.CommentType = PCDLRN.ENUM_PCD_COMMENT_TYPES.PCD_COMMENT_INPUT Then
                        txt = String.Format("{0} = {1}", Cmd.CommentCommand.Comment, Cmd.CommentCommand.Input)
                    End If

                    If Not String.IsNullOrEmpty(txt) Then
                        LineNum += 1

                        dr = dt.NewRow

                        dr.Item(1) = MisId
                        dr.Item(2) = LineNum
                        dr.Item(3) = "Commento"
                        dr.Item(4) = txt.Replace("'", "")
                        dr.Item(5) = 0
                        dr.Item(6) = 0
                        dr.Item(7) = 0
                        dr.Item(8) = 0
                        dr.Item(9) = 0
                        dr.Item(10) = 0
                        dr.Item(11) = 0
                        dr.Item(12) = False
                        dr.Item(13) = ""
                        dr.Item(14) = True
                        dr.Item(15) = Cmd.TypeDescription

                        dt.Rows.Add(dr)
                    End If
                End If

                'Import Dimensioni
                If Cmd.IsDimension Then

                    Dim DimCmd As PCDLRN.DimensionCmd = Cmd.DimensionCommand

                    If DimCmd.ID <> "" Then
                        DimID = DimCmd.ID   'Intestazione dimensione
                        LastDim = DimID
                        Dim txt As String = String.Format("DIM {0} - {1}", DimID, DimCmd.Feat1)

                        LineNum += 1

                        dr = dt.NewRow

                        dr.Item(1) = MisId
                        dr.Item(2) = LineNum
                        dr.Item(3) = "Intestazione"
                        dr.Item(4) = txt
                        dr.Item(5) = 0
                        dr.Item(6) = 0
                        dr.Item(7) = 0
                        dr.Item(8) = 0
                        dr.Item(9) = 0
                        dr.Item(10) = 0
                        dr.Item(11) = 0
                        dr.Item(12) = False
                        dr.Item(13) = ""
                        dr.Item(14) = True
                        dr.Item(15) = Cmd.TypeDescription

                        dt.Rows.Add(dr)

                        If Cmd.Type <> PCDLRN.OBTYPE.DIMENSION_START_LOCATION AndAlso Cmd.Type <> PCDLRN.OBTYPE.DIMENSION_TRUE_START_POSITION Then

                            Dim AxisLetter As String = DimCmd.AxisLetter
                            Dim MisValue As Double = DimCmd.Measured
                            Dim NomValue As Double = DimCmd.NOMINAL
                            Dim DevValue As Double = DimCmd.Deviation
                            Dim OutValue As Double = DimCmd.OutTol
                            Dim PlusTolValue As Double = DimCmd.Plus
                            Dim MinusTolValue As Double = DimCmd.Minus * -1
                            Dim BonusValue As Double = DimCmd.Bonus

                            Dim Verificato As Boolean = False

                            If My.Settings.AutoSpunta = True Then
                                Verificato = (OutValue = 0)
                            End If

                            Dim Nota As String = ""
                            dr = dt.NewRow

                            LineNum += 1

                            dr.Item(1) = MisId
                            dr.Item(2) = LineNum
                            dr.Item(3) = "Dim"
                            dr.Item(4) = LastDim & "_" & AxisLetter
                            dr.Item(5) = MisValue
                            dr.Item(6) = NomValue
                            dr.Item(7) = PlusTolValue
                            dr.Item(8) = MinusTolValue
                            dr.Item(9) = DevValue
                            dr.Item(10) = OutValue
                            dr.Item(11) = BonusValue
                            dr.Item(12) = Verificato
                            dr.Item(13) = Nota
                            dr.Item(14) = True
                            dr.Item(15) = Cmd.TypeDescription

                            dt.Rows.Add(dr)
                        End If

                    ElseIf Cmd.Type = PCDLRN.OBTYPE.DIMENSION_END_LOCATION Or Cmd.Type = PCDLRN.OBTYPE.DIMENSION_TRUE_END_POSITION Then
                        'non fa niente e va al prossimo elemento
                    Else

                        Dim AxisLetter As String = DimCmd.AxisLetter
                        Dim MisValue As Double = DimCmd.Measured
                        Dim NomValue As Double = DimCmd.NOMINAL
                        Dim DevValue As Double = DimCmd.Deviation
                        Dim OutValue As Double = DimCmd.OutTol
                        Dim PlusTolValue As Double = DimCmd.Plus
                        Dim MinusTolValue As Double = DimCmd.Minus * -1
                        Dim BonusValue As Double = DimCmd.Bonus

                        Dim Verificato As Boolean = False

                        If My.Settings.AutoSpunta = True Then
                            Verificato = (OutValue = 0)
                        End If

                        Dim Nota As String = ""
                        dr = dt.NewRow

                        LineNum += 1

                        dr.Item(1) = MisId
                        dr.Item(2) = LineNum
                        dr.Item(3) = "Dim"
                        dr.Item(4) = LastDim & "_" & AxisLetter
                        dr.Item(5) = MisValue
                        dr.Item(6) = NomValue
                        dr.Item(7) = PlusTolValue
                        dr.Item(8) = MinusTolValue
                        dr.Item(9) = DevValue
                        dr.Item(10) = OutValue
                        dr.Item(11) = BonusValue
                        dr.Item(12) = Verificato
                        dr.Item(13) = Nota
                        dr.Item(14) = True
                        dr.Item(15) = Cmd.TypeDescription

                        dt.Rows.Add(dr)
                    End If
                End If

            Next
        Finally

            'Memorizza i risultati
            Dim Msg As String
            Dim dta As New EasyRun.EasyRunDataSetTableAdapters.tbValoriTableAdapter

            'Estrae il MisID dal datatable corrente
            MisId = dt.Rows(0).Item(1).ToString()

            Dim CheckMis As Nullable(Of Integer) = dta.CheckExistMeasID(MisId)

            If CheckMis.HasValue Then
                Msg = (String.Format("{0}: #{1} Righe esistenti nel database per MisID {2}.", Date.Now, CheckMis, MisId) & ControlChars.CrLf)
                LogMsg.Invoke(Msg)
                Dim DeletedRows As Integer = dta.DeleteByMisID(MisId)

                Msg = (String.Format("{0}: #{1} Righe con MisID {2} eliminate per evitare duplicati.", Date.Now, DeletedRows, MisId) & ControlChars.CrLf)
                LogMsg.Invoke(Msg)
            End If

            Msg = (String.Format("{0}: #{1} Righe identificate per MisID {2} nel PartProgram {3}", Date.Now, dt.Rows.Count, MisId, Task.FileProgramma) & ControlChars.CrLf)
            LogMsg.Invoke(Msg)

            'Accoda le righe della Datatable al database
            Dim ImportedRows As Integer = dta.Update(dt)

            If ImportedRows > 0 Then
                Msg = (String.Format("{0}: #{1} Righe importate per MisID {2}", Date.Now, ImportedRows, MisId) & ControlChars.CrLf)
                LogMsg.Invoke(Msg)

            Else
                Msg = (String.Format("{0}: Nessuna riga identificata in {1}", Date.Now, Task.FileProgramma) & ControlChars.CrLf)
                LogMsg.Invoke(Msg)
            End If

        End Try
    End Sub
End Class
